package com.googlecode.jpattern.gwt.client.presenter;

import java.util.List;

import com.googlecode.jpattern.gwt.client.view.IShowViewStrategy;
import com.googlecode.jpattern.gwt.client.view.IView;
import com.googlecode.jpattern.shared.result.IErrorMessage;

/**
 * 
 * @author Francesco Cina'
 *
 * 14 Apr 2011
 */

public abstract class APresenter<T extends IView> implements IPresenter {
	
	private final IShowViewStrategy<T> showViewStrategy;

	public APresenter(IShowViewStrategy<T> showViewStrategy) {
		this.showViewStrategy = showViewStrategy;
		showViewStrategy.getView().visit(this);
		preDisplay();
		showViewStrategy.onLoadStart();
	    postDisplay();
	}
	
	@Override
	public final void onEventError(List<IErrorMessage> errorMessages) {
		showViewStrategy.getView().getErrorArea().addErrorMessages(errorMessages);
	}
	
	@Override
	public final void onEventStart() {
		showViewStrategy.getView().getErrorArea().clear();
	}
	
	protected final T getView() {
		return showViewStrategy.getView();
	}
	
	/**
	 * This method is called before the rendered of the IView.
	 * It should be better to not perform asynchronous actions in this method.
	 */
	public abstract void preDisplay();
	
	/**
	 * This method is called after the rendered of the IView.
	 * Here the asynchronous actions should be performed.
	 * Its VERY important to call the endLoading() method when all the actions are performed otherwise
	 * the IView should not be displayed. 
	 */
	public abstract void postDisplay();

	/**
	 * This method must be called when all the necessary operations to load necessary data are endend.
	 * This causes the final rendering of the IView. 
	 */
	public final void endLoading() {
		showViewStrategy.onLoadCompleted();
	}
	
}
